--[[
	## Interface: 70000

	## Title: HideHealingText
	## Notes: Filters healing received from the Blizzard Floating Combat Text
	## Author: Sav (weriK)

	v3.0
--]]

local _G = _G;
local PLAYER_NAME = UnitName("player");


local SAV_HHT_DEBUG = false;


-- EVENT HANDLING
--
--
--

-- First we need to create a frame to capture events
local SAV_HHTFrame = CreateFrame("FRAME");

-- Opt-in to receive the ADDON_LOADED event ( fired when all saved variables are loaded by the game )
SAV_HHTFrame:RegisterEvent("ADDON_LOADED");

-- This handles the events our frame receives
function SAV_HHTFrame:OnEvent(event, arg1)

	if event == "ADDON_LOADED" and arg1 == "HideHealingText" then
		
		-- Check if our saved variable exsists, if not we create a new one, and make sure the addon is turned off by default
		SAV_HHT_Load_SavedVars();

		if SAV_HHTState["firsttime"] then
			print("Welcome to HideHealingText!");
			SAV_HHT_Print_Help();
			SAV_HHTState["firsttime"] = false;
		end
		
		-- This is where the magic happens :)
		SAV_HHT_HookBlizzardCombatTextOnEvent();		

		SAV_HHT_Print_State_Info();
		
	end
end

function SAV_HHT_Load_SavedVars()

	if not SAV_HHTState then SAV_HHTState = {}; end;
		
	if  SAV_HHTState["firsttime"] 		== nil then SAV_HHTState["firsttime"] 		= true; end;
	if  SAV_HHTState["filterlimit"] 	== nil then SAV_HHTState["filterlimit"] 	= 0; end;
	if  SAV_HHTState["filterselflimit"] == nil then SAV_HHTState["filterselflimit"] = 0; end;

end

-- Set the event handler function for our frame
SAV_HHTFrame:SetScript("OnEvent", SAV_HHTFrame.OnEvent);


-- SLASH COMMAND
--
--
--

-- Define our slash commands
SLASH_SAV_HHT1 = "/hht";
SLASH_SAV_HHT2 = "/hidehealingtext";

-- Set a function we will execute when the slash command is used
SlashCmdList["SAV_HHT"] = function(msg, editbox)

	local command, arg = msg:match("^(%S+)%s*(%S-)$");
	arg = tonumber(arg);
	
	SAV_HHT_PRINT_DEBUG("Command:", command, "Arg:", arg);
	
	if command == "limit" and arg ~= nil then
		SAV_HHTState["filterlimit"] = math.max(arg, 0);
		SAV_HHT_Print_State_Info();
		
	elseif command == "selflimit" and arg ~= nil then
		SAV_HHTState["filterselflimit"] = math.max(arg, 0);
		SAV_HHT_Print_State_Info();
		
	elseif command == "debug" then
		SAV_HHT_DEBUG = not SAV_HHT_DEBUG;
	else
		SAV_HHT_Print_Help();
	end
	
end


-- PRINTOUTS
--
--
--

function SAV_HHT_Print_Help()
	
	print(" ");
	SAV_HHT_Print_State_Info()
	print("-- /hht limit <healing_amount> - Hides heals below this number. 0 to disable.");
	print("-- /hht selflimit <healing_amount> - Hides self-heals below this number. 0 to disable.");
	print("-- /hht - Show this help.");
	print(" ");

end


function SAV_HHT_Print_State_Info()
	
	msg = "";
	
	if 0 < SAV_HHTState["filterselflimit"] then
		msg = msg.."Self-healing below |cFFFF0000"..SAV_HHTState["filterselflimit"].."|r is |cFFFF0000HIDDEN|r,";
	else
		msg = msg.."All self-healing is |cFF00FF00VISIBLE|r,";
	end
	
	if 0 < SAV_HHTState["filterlimit"] then
		msg = msg.." heals below |cFFFF0000"..SAV_HHTState["filterlimit"].."|r are |cFFFF0000HIDDEN|r!";
	else
		msg = msg.." there is |cFF00FF00NO HEALING LIMIT|r for filtering!";
	end	
	
	print("HideHealingText:", msg);

end


-- MAIN
--
--
--

-- Is this a healing related combat text event?
function SAV_HHT_Is_Healing_Combat_Text_Event(event, combat_text_type)

	if ( event == "COMBAT_TEXT_UPDATE" 
	 and ( combat_text_type == "HEAL" 
		or combat_text_type == "PERIODIC_HEAL"
		or combat_text_type == "HEAL_ABSORB" 
		or combat_text_type == "PERIODIC_HEAL_ABSORB"
		or combat_text_type == "HEAL_CRIT" 
		or combat_text_type == "PERIODIC_HEAL_CRIT"
		or combat_text_type == "HEAL_CRIT_ABSORB"
		or combat_text_type == "ABSORB_ADDED" )) then 
			return true;
	else
		return false;
	end
	
end

-- Is the self-healing amount below the filtering limit?
function SAV_HHT_Is_Self_Healing_Below_Limit(healing_amount)

	SAV_HHT_PRINT_DEBUG("Self-healing limit", SAV_HHTState["filterselflimit"], "Healing amount:", healing_amount);
	return healing_amount <= SAV_HHTState["filterselflimit"];
	
end

-- Is the healing amount below the filtering limit?
function SAV_HHT_Is_Healing_Amount_Below_Limit(healing_amount)

	SAV_HHT_PRINT_DEBUG("Healing limit", SAV_HHTState["filterlimit"], "Healing amount:", healing_amount);
	return healing_amount <= SAV_HHTState["filterlimit"];
	
end

-- Should we hide the event?!
function SAV_HHT_Should_Hide_Event(target_unit, healing_amount)

	if PLAYER_NAME == target_unit then
		if SAV_HHT_Is_Self_Healing_Below_Limit(healing_amount) then
			SAV_HHT_PRINT_DEBUG("Self-healing filtered");
			return true;
		else
			return false;
		end
		
	elseif SAV_HHT_Is_Healing_Amount_Below_Limit(healing_amount) then
		SAV_HHT_PRINT_DEBUG("Healing filtered");
		return true;
	else
		SAV_HHT_PRINT_DEBUG("Healing not filtered");
		return false;
	end
	
end

-- Hook the Blizzard CombatText_OnEvent
function SAV_HHT_HookBlizzardCombatTextOnEvent()

	-- Save the original event handler
	local original = _G.CombatText_OnEvent;
	
	-- Override the original event handler with our own hook
	CombatText:SetScript("OnEvent", function(self, event, ...)
		
	local combat_text_type, target_unit, healing_amount, _ = ...;
--		SAV_HHT_PRINT_DEBUG(event, "|", combat_text_type, "|", target_unit, "|", healing_amount);
	
	if   SAV_HHT_Is_Healing_Combat_Text_Event(event, combat_text_type)
	 and SAV_HHT_Should_Hide_Event(target_unit, healing_amount) then
			-- Hide this heal!
			SAV_HHT_PRINT_DEBUG(event, "|", combat_text_type, "|", target_unit, "|", healing_amount);
			return;
	else
		-- Call the original event handler as if nothing happened
		return original(self, event, ...);			
	end
		
	end);
	
end

function SAV_HHT_PRINT_DEBUG(...)
	if SAV_HHT_DEBUG then
		print("HHT_DEBUG:", ...);
	end
end